package parser;

import genvisitor.*;

/**
 * Factory for E1a grammar non-terminal objects.
 */
public class E1aFact extends ATVFactory {
    /**
     * Factory for E grammar non-terminals.
     */
    private EFact _eFact;

    /**
     * Visitor for E grammar non-terminals.
     */
    private ATokVisitor<IGrammarToken, Object> _parseE;

    /**
     * Initializer lambda for this factory.
     */
    private ILambda _initializer = new ILambda() {
        public Object apply(Object param) {
            // change state to no-op
            _initializer = NoOpLambda.Singleton;

            // initialize
            _parseE = _eFact.makeVisitor();
            return null;
        }
    };

    /**
     * Constructor for the E1a factory.
     *
     * @param tkz   tokenizer to use
     * @param eFact factory for E grammar non-terminals
     */
    public E1aFact(ITokenizer tkz, EFact eFact) {
        super(tkz);
        _eFact = eFact;
    }

    /**
     * Constructor for the E1a factory, Does not take a factory for E grammar non-terminals. 
     * That factory has to be set using setParseEFactory().
     *
     * @param tkz tokenizer to use
     */
    public E1aFact(ITokenizer tkz) {
        super(tkz);
    }

    /**
     * Set the factory for E grammar non-terminals.
     *
     * @param eFact factory for E grammar non-terminals
     */
    public void setParseEFactory(EFact eFact) {
        _eFact = eFact;
    }

    private void addCmdTo(ATokVisitor<IGrammarToken, Object> target) {
      target.addCmd("+", new AGenVisitor.IGenVisitorCmd<IGrammarToken, String, Object, AToken>() {
              public IGrammarToken apply(String idx, AToken host, Object... inp) {
                return new E1a((PlusToken)host, (E) nextToken().execute(_parseE, inp));
              }
            });
    }

    /**
     * Make a token visitor to parse an E1a non-terminal.
     *
     * @return token visitor
     */
    public ATokVisitor<IGrammarToken, Object> makeVisitor() {
        initialize();

        return new ATokVisitor<IGrammarToken, Object>() {
          {
            addCmdTo(this);
          }
        };
    }

    /**
     * Make a token visitor that delegates to the given visitor in a chain of responsibility
     *
     * @param successor visitor to serve as successor in the chain
     */
    public ATokVisitor<IGrammarToken, Object> makeChainedVisitor(ATokVisitor<IGrammarToken, Object> successor) {
        initialize();

        return new ATokVisitor<IGrammarToken, Object>(successor){
          {
            addCmdTo(this);
          }
        };
    }

    /**
     * Make the visitor.
     */
    private void initialize() {
        _initializer.apply(null);

    }
}

